<?php

namespace Services;


use Exception\CustomException;
use Util\RepositoryInstance;
use Util\Utilities;
use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Util\Messages as Messages;
use Factory\ParamOrg as ParamOrgFactory;
use Factory\ParamSenha as ParamSenhaFactory;
use Factory\IpRestriction as IpRestrictionFactory;
use Teknisa\Model\Geral\Repositories\Paramsenha as ParamsenhaRepository;
use Teknisa\Model\Teknisa\Repositories\OmParamOrg as ParamOrgRepository;

class Settings {
    /** @var EntityManager */
    private $entityManager;
    /** @var Environment */
    private $environment;
    /** @var  ParamsenhaRepository */
    private $paramSenhaRepository;
    /** @var  ParamOrgRepository */
    private $paramOrgRepository;
    /** @var  IpRestrictionRepository */
    private $ipRestrictionRepository;
    /** @var ParamSenhaFactory */
    private $paramSenhaFactory;
    /** @var ParamOrgFactory */
    private $paramOrgFactory;
    /** @var IpRestrictionFactory */
    private $ipRestrictionFactory;

    /**
     * @param EntityManager $entityManager
     * @param Environment $environment
     * @param ParamSenhaFactory $paramSenhaFactory
     */
    public function __construct(EntityManager $entityManager, Environment $environment, ParamSenhaFactory $paramSenhaFactory, ParamOrgFactory $paramOrgFactory, IpRestrictionFactory $ipRestrictionFactory) {
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->paramOrgFactory = $paramOrgFactory;
        $this->paramSenhaFactory = $paramSenhaFactory;
        $this->ipRestrictionFactory = $ipRestrictionFactory;
        $this->paramOrgRepository = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmParamOrg');
        $this->paramSenhaRepository = $this->entityManager->getRepository(Utilities::GENERAL_ENTITY_NAMESPACE . 'Paramsenha');
        $this->ipRestrictionRepository = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmIpRestriction');
    }

    /**
     * @return array
     */
    public function findADConfigurationByNrOrg() {
        return $this->paramSenhaRepository->getADConfigurationByNrOrg($this->environment->getCurrentOrganizationId());
    }

    /**
     * @param $domain
     * @param $server
     * @param $admUser
     * @param $admPassword
     * @param $useAd
     * @throws \Doctrine\DBAL\ConnectionException
     * @throws \Exception
     * @return array
     */
    public function saveADConfiguration($domain, $server, $admUser, $admPassword, $useAd) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $paramSenha = $this->paramSenhaRepository->findOneBy(array('nrorg' => $this->environment->getCurrentOrganizationId()));
            $paramSenha = $this->paramSenhaFactory->saveADConfiguration($paramSenha, $this->environment->getCurrentOrganizationId(),
                $this->environment->getCurrentUserId(), $domain, $server, $admUser, $admPassword, $useAd);
            $this->entityManager->persist($paramSenha);
            $this->entityManager->flush();
            $connection->commit();
            return array('NRORG' => $paramSenha->getNrorg());
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function getOrganizationRestrictionByIp(){
        return RepositoryInstance::getOmParamOrgLocal()->getOrganizationRestrictionByIp($this->environment->getCurrentOrganizationId());
    }
    
    public function getOrganizationParameters (){
        $orgId = $this->environment->getCurrentOrganizationId();
        $parameters = $this->paramOrgRepository->getOrganizationParameters($orgId);
        if (!$parameters) {
            $parameters[0] = array('ORG_ID' => $orgId, 'USE_RESTRICTION' => 'N');
        }
        return $parameters;
    }
    
    public function saveOrganizationParameters($useRestriction, $idUtCtrlSen, $nrDiasExpSen,
                                               $nrDiasAviso, $nrAltSenhRep, $cdPictSenha, $nrErroSenha){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $dateNow      = new \DateTime();
            $codOperador  = $this->environment->getCurrentUserId();
            $currentOrgId = $this->environment->getCurrentOrganizationId();

            /** @var \Teknisa\Model\Teknisa\Entities\OmParamOrg $omParamOrg */
            $omParamOrg = RepositoryInstance::getOmParamOrg()->findOneBy(array('nrorg' => $currentOrgId));
            $omParamOrg = ParamOrgFactory::save($omParamOrg, $currentOrgId, $codOperador, $dateNow, $useRestriction);
            $this->entityManager->persist($omParamOrg);

            $paramSenha = RepositoryInstance::getParamSenha()->findOneBy(array('nrorg' => $currentOrgId));
            $paramSenha = ParamSenhaFactory::savePasswordParams($paramSenha, $idUtCtrlSen, $nrDiasExpSen,
                $nrDiasAviso, $nrAltSenhRep, $cdPictSenha, $nrErroSenha, $currentOrgId, $codOperador);
            $this->entityManager->persist($paramSenha);

            $this->entityManager->flush();
            $connection->commit();
        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function saveOrganizationRestrictionByIp($ipRestrictedId, $ipv4, $ipv6, $description, $active){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $dateNow = new \DateTime();
            $codOperador = $this->environment->getCurrentUserId();
            $orgId = $this->environment->getCurrentOrganizationId();
            $this->validateIps($ipv4, $ipv6);
            $ipv4 ? $ipv6 = null : $ipv4 = null;
            
            if ($ipRestrictedId){
                $savedRestriction = $this->updateRestriction($ipRestrictedId, $ipv4, $ipv6, $description, $active, $orgId,
                    $codOperador, $dateNow);
            } else {
                $savedRestriction = $this->createRestrictionIp($ipv4, $ipv6, $description, $active, $orgId, $codOperador, $dateNow);
            }

            if (!$savedRestriction){
                throw new CustomException('ERR_NOT_SAVED');
            }

            $this->entityManager->persist($savedRestriction);
            $this->entityManager->flush();
            $connection->commit();
            return array("ID" => $savedRestriction->getNrIpRestriction());
        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    private function createRestrictionIp($ipv4, $ipv6, $description, $active, $orgId, $codOperador, $dateNow){
        $this->checkIfIpAlreadyRegistered($orgId, $ipv4, $ipv6);
        return $this->ipRestrictionFactory->save(null, $orgId, $orgId, $codOperador, $dateNow, $description, $ipv4, $ipv6, $active);
    }
    
    private function checkIfIpAlreadyRegistered($orgId, $ipv4, $ipv6){
        if ($ipv4){
            if ($this->ipRestrictionRepository->findOneBy(array('nrorg' => $orgId, 'dsIpv4Restricted' => $ipv4))) {
                throw new CustomException('ERR_IP_ALREADY_REGISTERED');
            } 
        } else {
            if ($this->ipRestrictionRepository->findOneBy(array('nrorg' => $orgId, 'dsIpv6Restricted' => $ipv6))) {
                throw new CustomException('ERR_IP_ALREADY_REGISTERED');
            }
        }
    }
    
    public function removeOrganizationRestrictionByIp($restrictionId){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $orgId = $this->environment->getCurrentOrganizationId();
            if ($restrictionId) {
                $organizationRestriction = $this->ipRestrictionRepository->findOneBy(array('nrIpRestriction' => $restrictionId, 'nrorg' => $orgId));
                if ($organizationRestriction){
                    $this->entityManager->remove($organizationRestriction);
                    $this->entityManager->flush();
                    $this->entityManager->commit();
                }
            }
        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    private function isIpValid($ip, $ipVersion){
        if ($ipVersion == "IPV4"){
            return filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4) ? true : false;
        } else if ($ipVersion == "IPV6") {
            return filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV6) ? true : false;
        } else {
            return false;
        }
    }

    /**
     * @param $ipv4
     * @param $ipv6
     * @throws \Exception
     */
    private function validateIps($ipv4, $ipv6){
        if (!$ipv4 && !$ipv6) {
            throw new CustomException('ERR_NONE_IP_INFORMED');
        } else {
            if ($ipv4 && !$this->isIpValid($ipv4, "IPV4")) {
                throw new CustomException('ERR_IP_INVALID');
            } else if ($ipv6 && !$this->isIpValid($ipv6, "IPV6")) {
                throw new CustomException('ERR_IP_INVALID');
            }
        }
    }

    /**
     * @param $ipRestrictedId
     * @param $ipv4
     * @param $ipv6
     * @param $description
     * @param $active
     * @param $orgId
     * @param $codOperador
     * @param $dateNow
     * @return object
     * @throws \Exception
     */
    private function updateRestriction($ipRestrictedId, $ipv4, $ipv6, $description, $active, $orgId, $codOperador, $dateNow){
        $ipRestricted = $this->ipRestrictionRepository->findOneBy(array('nrorg' => $orgId, 'nrIpRestriction' => $ipRestrictedId));
        if ($ipRestricted) {
            if ($ipRestricted->getDsIpv4Restricted() == $ipv4 && $ipRestricted->getDsIpv6Restricted() == $ipv6){
                return $this->ipRestrictionFactory->save($ipRestricted, $orgId, $orgId, $codOperador, $dateNow,
                    $description, $ipv4, $ipv6, $active);
            } else {
                $this->checkIfIpAlreadyRegistered($orgId, $ipv4, $ipv6);
                return $this->ipRestrictionFactory->save($ipRestricted, $orgId, $orgId, $codOperador, $dateNow,
                    $description, $ipv4, $ipv6, $active);
            }
        } else {
            throw new CustomException('ERR_RESTRICTION_CHANGES_NOT_SAVED');
        }
    }

} 